import { createDisposable } from "@reactive-js/disposable";
import { schedule } from "@reactive-js/schedulers";
const performance = window.performance;
const Date = window.Date;
const setTimeout = window.setTimeout;
const clearTimeout = window.clearTimeout;
const now = typeof performance === "object" && typeof performance.now === "function"
    ? () => performance.now()
    : () => Date.now();
const yieldInterval = 5;
const maxYieldInterval = 300;
const callCallbackAndDispose = (scheduler, callback, disposable) => {
    scheduler.startTime = scheduler.now;
    callback();
    disposable.dispose();
};
const scheduleImmediate = (scheduler, callback) => {
    const disposable = createDisposable();
    const channel = scheduler.channel;
    channel.port1.onmessage = () => {
        if (!disposable.isDisposed) {
            scheduler.startTime = scheduler.now;
            callback();
            disposable.dispose();
        }
    };
    channel.port2.postMessage(null);
    return disposable;
};
const scheduleDelayed = (scheduler, callback, delay) => {
    const disposable = createDisposable(() => clearTimeout(timeout));
    const timeout = setTimeout(callCallbackAndDispose, delay, scheduler, callback, disposable);
    return disposable;
};
class WebScheduler {
    constructor() {
        this.channel = new MessageChannel();
        this.schedule = schedule;
        this.shouldYield = navigator !== undefined &&
            navigator.scheduling !== undefined &&
            navigator.scheduling.isInputPending !== undefined
            ? () => {
                const currentTime = now();
                const startTime = this.startTime;
                const deadline = startTime + yieldInterval;
                const maxDeadline = startTime + maxYieldInterval;
                const inputPending = navigator.scheduling.isInputPending();
                return ((currentTime >= deadline && inputPending) ||
                    currentTime >= maxDeadline);
            }
            : () => now() >= this.startTime + yieldInterval;
        this.startTime = this.now;
    }
    get now() {
        return now();
    }
    scheduleCallback(callback, delay) {
        return delay >= 4
            ? scheduleDelayed(this, callback, delay)
            : scheduleImmediate(this, callback);
    }
}
let hostScheduler = undefined;
export const getHostScheduler = () => {
    hostScheduler = hostScheduler || new WebScheduler();
    return hostScheduler;
};
//# sourceMappingURL=scheduler.js.map